@@ -33,6 +33,7 @@ from zigpy.quirks.v2.homeassistant.number import NumberDeviceClass
 from zigpy.quirks.v2.homeassistant.sensor import SensorDeviceClass, SensorStateClass
 import zigpy.types as t
 from zigpy.zcl import ClusterType
+from zigpy.zcl.clusters.general import Ota
 from zigpy.zdo import ZDO
 from zigpy.zdo.types import NodeDescriptor
 
@@ -455,6 +456,37 @@ class PreventDefaultEntityCreationMetadata:
     function: Callable[[Any], bool] | None = attrs.field()
 
 
+@attrs.define(frozen=True, kw_only=True, repr=True)
+class ChangedEntityMetadata:
+    """Metadata to change entity metadata for matching entities."""
+
+    endpoint_id: int | None = attrs.field()
+    cluster_id: int | None = attrs.field()
+    cluster_type: ClusterType | None = attrs.field()
+    unique_id_suffix: str | None = attrs.field()
+    function: Callable[[Any], bool] | None = attrs.field()
+    # Entity metadata changes
+    new_primary: bool | None = attrs.field(default=None)
+    new_unique_id: str | None = attrs.field(default=None)
+    new_translation_key: str | None = attrs.field(default=None)
+    new_device_class: (
+        BinarySensorDeviceClass | NumberDeviceClass | SensorDeviceClass | None
+    ) = attrs.field(default=None)
+    new_state_class: SensorStateClass | None = attrs.field(default=None)
+    new_entity_category: EntityType | None = attrs.field(default=None)
+    new_entity_registry_enabled_default: bool | None = attrs.field(default=None)
+    new_fallback_name: str | None = attrs.field(default=None)
+
+
+@attrs.define(frozen=True, kw_only=True, repr=True)
+class FirmwareVersionFilterMetadata:
+    """Metadata to only apply the quirk if the device's firmware version matches."""
+
+    min_version: int | None = attrs.field(default=None)
+    max_version: int | None = attrs.field(default=None)
+    allow_missing: bool = attrs.field(default=True)
+
+
 @attrs.define(frozen=True, kw_only=True, repr=True)
 class QuirksV2RegistryEntry:
     """Quirks V2 registry entry."""
@@ -469,7 +501,9 @@ class QuirksV2RegistryEntry:
     disabled_default_entities: tuple[PreventDefaultEntityCreationMetadata] = (
         attrs.field(factory=tuple)
     )
+    changed_entity_metadata: tuple[ChangedEntityMetadata] = attrs.field(factory=tuple)
     filters: tuple[FilterType] = attrs.field(factory=tuple)
+    fw_version_filter: FirmwareVersionFilterMetadata | None = attrs.field(default=None)
     custom_device_class: type[CustomDeviceV2] | None = attrs.field(default=None)
     device_node_descriptor: NodeDescriptor | None = attrs.field(default=None)
     skip_device_configuration: bool = attrs.field(default=False)
@@ -500,7 +534,33 @@ class QuirksV2RegistryEntry:
 
     def matches_device(self, device: Device) -> bool:
         """Determine if this quirk should be applied to the passed in device."""
-        return all(_filter(device) for _filter in self.filters)
+        if not all(_filter(device) for _filter in self.filters):
+            return False
+
+        if self.fw_version_filter is not None:
+            try:
+                ota = device.find_cluster(
+                    cluster_id=Ota.cluster_id, cluster_type=ClusterType.Client
+                )
+            except ValueError:
+                return self.fw_version_filter.allow_missing
+
+            current_file_version = ota.get(Ota.AttributeDefs.current_file_version.id)
+
+            if current_file_version is None:
+                return self.fw_version_filter.allow_missing
+
+            if self.fw_version_filter.min_version is not None and (
+                current_file_version < self.fw_version_filter.min_version
+            ):
+                return False
+
+            if self.fw_version_filter.max_version is not None and (
+                current_file_version >= self.fw_version_filter.max_version
+            ):
+                return False
+
+        return True
 
     def create_device(self, device: Device) -> CustomDeviceV2:
         """Create the quirked device."""
@@ -531,7 +591,9 @@ class QuirkBuilder:
         self.friendly_name_metadata: FriendlyNameMetadata | None = None
         self.device_alerts: list[DeviceAlertMetadata] = []
         self.disabled_default_entities: list[PreventDefaultEntityCreationMetadata] = []
+        self.changed_entity_metadata: list[ChangedEntityMetadata] = []
         self.filters: list[FilterType] = []
+        self.fw_version_filter: FirmwareVersionFilterMetadata | None = None
         self.custom_device_class: type[CustomDeviceV2] | None = None
         self.device_node_descriptor: NodeDescriptor | None = None
         self.skip_device_configuration: bool = False
@@ -599,6 +661,25 @@ class QuirkBuilder:
         self.filters.append(filter_function)
         return self
 
+    def firmware_version_filter(
+        self,
+        min_version: int | None = None,
+        max_version: int | None = None,
+        allow_missing: bool = True,
+    ) -> QuirkBuilder:
+        """Add a firmware version filter and returns self.
+
+        The min_version and max_version are integers representing the firmware version,
+        minimum inclusive but maximum exclusive. If allow_missing is True, the filter
+        will pass if the device does not have a firmware version.
+        """
+        self.fw_version_filter = FirmwareVersionFilterMetadata(
+            min_version=min_version,
+            max_version=max_version,
+            allow_missing=allow_missing,
+        )
+        return self
+
     def device_class(self, custom_device_class: type[CustomDeviceV2]) -> QuirkBuilder:
         """Set the custom device class to be used in this quirk and returns self.
 
@@ -1127,6 +1208,48 @@ class QuirkBuilder:
         )
         return self
 
+    def change_entity_metadata(
+        self,
+        *,
+        endpoint_id: int | None = None,
+        cluster_id: int | None = None,
+        cluster_type: ClusterType | None = None,
+        unique_id_suffix: str | None = None,
+        function: Callable[[Any], bool] | None = None,
+        new_primary: bool | None = None,
+        new_unique_id: str | None = None,
+        new_translation_key: str | None = None,
+        new_device_class: (
+            BinarySensorDeviceClass | NumberDeviceClass | SensorDeviceClass | None
+        ) = None,
+        new_state_class: SensorStateClass | None = None,
+        new_entity_category: EntityType | None = None,
+        new_entity_registry_enabled_default: bool | None = None,
+        new_fallback_name: str | None = None,
+    ) -> QuirkBuilder:
+        """Change entity metadata for matching entities."""
+        if cluster_id is not None and cluster_type is None:
+            cluster_type = ClusterType.Server
+
+        self.changed_entity_metadata.append(
+            ChangedEntityMetadata(
+                endpoint_id=endpoint_id,
+                cluster_id=cluster_id,
+                cluster_type=cluster_type,
+                unique_id_suffix=unique_id_suffix,
+                function=function,
+                new_primary=new_primary,
+                new_unique_id=new_unique_id,
+                new_translation_key=new_translation_key,
+                new_device_class=new_device_class,
+                new_state_class=new_state_class,
+                new_entity_category=new_entity_category,
+                new_entity_registry_enabled_default=new_entity_registry_enabled_default,
+                new_fallback_name=new_fallback_name,
+            ),
+        )
+        return self
+
     def add_to_registry(self) -> QuirksV2RegistryEntry:
         """Build the quirks v2 registry entry."""
         if not self.manufacturer_model_metadata:
@@ -1138,9 +1261,11 @@ class QuirkBuilder:
             friendly_name=self.friendly_name_metadata,
             device_alerts=tuple(self.device_alerts),
             disabled_default_entities=tuple(self.disabled_default_entities),
+            changed_entity_metadata=tuple(self.changed_entity_metadata),
             quirk_file=self.quirk_file,
             quirk_file_line=self.quirk_file_line,
             filters=tuple(self.filters),
+            fw_version_filter=self.fw_version_filter,
             custom_device_class=self.custom_device_class,
             device_node_descriptor=self.device_node_descriptor,
             skip_device_configuration=self.skip_device_configuration,
