@@ -4,6 +4,7 @@ import abc
 import asyncio
 import functools
 import inspect
+import itertools
 import logging
 import traceback
 import typing
@@ -15,11 +16,13 @@ from cryptography.hazmat.primitives.ciphers.algorithms import AES
 from cryptography.hazmat.primitives.ciphers.modes import ECB
 
 from zigpy.datastructures import DynamicBoundedSemaphore  # noqa: F401
-from zigpy.exceptions import ControllerException, ZigbeeException
+from zigpy.exceptions import ZigbeeException
 import zigpy.types as t
 
 LOGGER = logging.getLogger(__name__)
 
+_T = typing.TypeVar("_T")
+
 
 class ListenableMixin:
     def __init__(self, *args, **kwargs) -> None:
@@ -47,10 +50,10 @@ class ListenableMixin:
 
     def listener_event(self, method_name: str, *args) -> list[typing.Any | None]:
         result = []
-        for listener, include_context in self._listeners.values():
+        for listener, include_context in tuple(self._listeners.values()):
             method = getattr(listener, method_name, None)
 
-            if not method:
+            if method is None:
                 continue
 
             try:
@@ -58,7 +61,7 @@ class ListenableMixin:
                     result.append(method(self, *args))
                 else:
                     result.append(method(*args))
-            except Exception as e:
+            except Exception as e:  # noqa: BLE001
                 LOGGER.debug(
                     "Error calling listener %r with args %r", method, args, exc_info=e
                 )
@@ -66,10 +69,10 @@ class ListenableMixin:
 
     async def async_event(self, method_name: str, *args) -> list[typing.Any]:
         tasks = []
-        for listener, include_context in self._listeners.values():
+        for listener, include_context in tuple(self._listeners.values()):
             method = getattr(listener, method_name, None)
 
-            if not method:
+            if method is None:
                 continue
 
             if include_context:
@@ -119,7 +122,7 @@ async def retry(
     func: typing.Callable[[], typing.Awaitable[typing.Any]],
     retry_exceptions: typing.Iterable[BaseException],
     tries: int = 3,
-    delay: int | float = 0.1,
+    delay: float = 0.1,
 ) -> typing.Any:
     """Retry a function in case of exception
 
@@ -247,51 +250,6 @@ def convert_install_code(code: bytes) -> t.KeyData:
     return aes_mmo_hash(code)
 
 
-T = typing.TypeVar("T")
-
-
-class Request(typing.Generic[T]):
-    """Request context manager."""
-
-    def __init__(self, pending: dict, sequence: T) -> None:
-        """Init context manager for requests."""
-        self._pending = pending
-        self._result: asyncio.Future = asyncio.Future()
-        self._sequence = sequence
-
-    @property
-    def result(self) -> asyncio.Future:
-        return self._result
-
-    @property
-    def sequence(self) -> T:
-        """Request sequence."""
-        return self._sequence
-
-    def __enter__(self) -> Request:
-        """Return context manager."""
-        self._pending[self.sequence] = self
-        return self
-
-    def __exit__(self, exc_type: None, exc_value: None, exc_traceback: None) -> bool:
-        """Clean up pending on exit."""
-        if not self.result.done():
-            self.result.cancel()
-        self._pending.pop(self.sequence)
-
-        return not exc_type
-
-
-class Requests(dict, typing.Generic[T]):
-    def new(self, sequence: T) -> Request[T]:
-        """Wrap new request into a context manager."""
-        if sequence in self:
-            LOGGER.debug("Duplicate %s TSN: pending %s", sequence, self)
-            raise ControllerException(f"Duplicate TSN: {sequence}")
-
-        return Request(self, sequence)
-
-
 class CatchingTaskMixin(LocalLogMixin):
     """Allow creating tasks suppressing exceptions."""
 
@@ -350,7 +308,7 @@ def deprecated(message: str) -> typing.Callable[[typing.Callable], typing.Callab
 
 
 def deprecated_attrs(
-    mapping: dict[str, typing.Any]
+    mapping: dict[str, typing.Any],
 ) -> typing.Callable[[str], typing.Any]:
     """Create a module-level `__getattr__` function that remaps deprecated objects."""
 
@@ -402,7 +360,7 @@ def pick_optimal_channel(
 
     # Scan all channels even if we're restricted to picking among a few, since
     # nearby channels will affect our decision
-    assert set(channel_energy.keys()) == set(t.Channels.ALL_CHANNELS)  # type: ignore
+    assert set(channel_energy.keys()) == set(t.Channels.ALL_CHANNELS)  # type: ignore[call-overload]
 
     # We don't know energies above channel 26 or below 11. Assume the scan results
     # just continue indefinitely with the last-seen value.
@@ -450,3 +408,65 @@ class Singleton:
 
     def __hash__(self) -> int:
         return hash(self.name)
+
+
+def filter_relays(relays: list[int]) -> list[int]:
+    """Filter out invalid relays."""
+    filtered_relays = []
+
+    # BUG: relays sometimes include 0x0000 or duplicate entries
+    for relay in relays:
+        if relay != 0x0000 and relay not in filtered_relays:
+            filtered_relays.append(relay)
+
+    return filtered_relays
+
+
+def combine_concurrent_calls(
+    function: typing.Callable[
+        ..., typing.Coroutine[typing.Any, typing.Any, typing.Any]
+    ],
+) -> typing.Callable[..., typing.Coroutine[typing.Any, typing.Any, typing.Any]]:
+    """Decorator that allows concurrent calls to expensive coroutines to share a result."""
+
+    tasks: dict[tuple, asyncio.Task] = {}
+    signature = inspect.signature(function)
+
+    @functools.wraps(function)
+    async def replacement(*args: typing.Any, **kwargs: typing.Any) -> typing.Any:
+        bound = signature.bind(*args, **kwargs)
+        bound.apply_defaults()
+
+        # XXX: all args and kwargs are assumed to be hashable
+        key = tuple(bound.arguments.items())
+
+        if key in tasks:
+            return await tasks[key]
+
+        tasks[key] = asyncio.create_task(function(*args, **kwargs))
+
+        try:
+            return await tasks[key]
+        finally:
+            assert tasks[key].done()
+            del tasks[key]
+
+    return replacement
+
+
+async def async_iterate_in_chunks(
+    iterable: typing.Iterable[_T], chunk_size: int
+) -> typing.AsyncGenerator[list[_T], None]:
+    """Safely iterate over a synchronous iterable in chunks."""
+    loop = asyncio.get_running_loop()
+    iterator = iter(iterable)
+
+    while True:
+        chunk = await loop.run_in_executor(
+            None, list, itertools.islice(iterator, chunk_size)
+        )
+
+        if not chunk:
+            break
+
+        yield chunk
