@@ -369,19 +369,23 @@ class Routes(t.Struct):
     RoutingTableList: t.LVList[Route]
 
 
+CHANNEL_CHANGE_REQ = 0xFE
+CHANNEL_MASK_MANAGER_ADDR_CHANGE_REQ = 0xFF
+
+
 class NwkUpdate(t.Struct):
-    CHANNEL_CHANGE_REQ = 0xFE
-    CHANNEL_MASK_MANAGER_ADDR_CHANGE_REQ = 0xFF
+    CHANNEL_CHANGE_REQ = CHANNEL_CHANGE_REQ
+    CHANNEL_MASK_MANAGER_ADDR_CHANGE_REQ = CHANNEL_MASK_MANAGER_ADDR_CHANGE_REQ
 
     ScanChannels: t.Channels
     ScanDuration: t.uint8_t
     ScanCount: t.uint8_t = t.StructField(requires=lambda s: s.ScanDuration <= 0x05)
-    nwkUpdateId: t.uint8_t = t.StructField(
+    nwkUpdateId: t.uint8_t = t.StructField(  # noqa: N815
         requires=lambda s: s.ScanDuration
-        in (s.CHANNEL_CHANGE_REQ, s.CHANNEL_MASK_MANAGER_ADDR_CHANGE_REQ)
+        in (CHANNEL_CHANGE_REQ, CHANNEL_MASK_MANAGER_ADDR_CHANGE_REQ)
     )
-    nwkManagerAddr: t.NWK = t.StructField(
-        requires=lambda s: s.ScanDuration == s.CHANNEL_MASK_MANAGER_ADDR_CHANGE_REQ
+    nwkManagerAddr: t.NWK = t.StructField(  # noqa: N815
+        requires=lambda s: s.ScanDuration == CHANNEL_MASK_MANAGER_ADDR_CHANGE_REQ
     )
 
 
@@ -717,7 +721,7 @@ CLUSTERS = {
         ("TotalTransmissions", t.uint16_t),
         ("TransmissionFailures", t.uint16_t),
         ("EnergyValues", t.LVList[t.uint8_t]),
-    )
+    ),
     # ... TODO optional stuff ...
 }
 
